@@ -48,14 +48,14 @@ indicated above, though the model DOES NOT have to be named
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 import os
 import sys
 
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 # pylint: disable=import-error
 try:
     import django
@@ -64,7 +64,7 @@ try:
 except Exception as exc:
     # If Django is installed and is not detected, uncomment
     # the following line to display additional information
-    #log.warning('Could not load Django auth module. Found exception: {0}'.format(exc))
+    #log.warning('Could not load Django auth module. Found exception: %s', exc)
     HAS_DJANGO = False
 # pylint: enable=import-error
 
@@ -90,7 +90,7 @@ def is_connection_usable():
         return True
 
 
-def django_auth_setup():
+def __django_auth_setup():
     '''
     Prepare the connection to the Django authentication framework
     '''
@@ -125,32 +125,29 @@ def auth(username, password):
         sys.path.append(django_auth_path)
     os.environ.setdefault('DJANGO_SETTINGS_MODULE', __opts__['django_auth_settings'])
 
-    django_auth_setup()
+    __django_auth_setup()
 
     if not is_connection_usable():
         connection.close()
 
-    import django.contrib.auth  # pylint: disable=import-error
+    import django.contrib.auth  # pylint: disable=import-error,3rd-party-module-not-gated
     user = django.contrib.auth.authenticate(username=username, password=password)
     if user is not None:
         if user.is_active:
             log.debug('Django authentication successful')
-
-            auth_dict_from_db = retrieve_auth_entries(username)[username]
-            if auth_dict_from_db is not None:
-                return auth_dict_from_db
-
             return True
         else:
             log.debug('Django authentication: the password is valid but the account is disabled.')
+    else:
+        log.debug('Django authentication failed.')
 
     return False
 
 
-def retrieve_auth_entries(u=None):
+def acl(username):
     '''
 
-    :param u: Username to filter for
+    :param username: Username to filter for
     :return: Dictionary that can be slotted into the ``__opts__`` structure for
         eauth that designates the user associated ACL
 
@@ -185,12 +182,12 @@ def retrieve_auth_entries(u=None):
             - .*
 
     '''
-    django_auth_setup()
+    __django_auth_setup()
 
-    if u is None:
+    if username is None:
         db_records = DJANGO_AUTH_CLASS.objects.all()
     else:
-        db_records = DJANGO_AUTH_CLASS.objects.filter(user_fk__username=u)
+        db_records = DJANGO_AUTH_CLASS.objects.filter(user_fk__username=username)
     auth_dict = {}
 
     for a in db_records:
@@ -211,5 +208,5 @@ def retrieve_auth_entries(u=None):
             if not found:
                 auth_dict[a.user_fk.username].append({a.minion_or_fn_matcher: [a.minion_fn]})
 
-    log.debug('django auth_dict is {0}'.format(repr(auth_dict)))
+    log.debug('django auth_dict is %s', auth_dict)
     return auth_dict
