@@ -87,7 +87,7 @@ def _get_username(member):
         str: The username converted to domain\\username format
     '''
     return member.ADSPath.replace('WinNT://', '').replace(
-        '/', '\\').encode('ascii', 'backslashreplace')
+        '/', '\\')
 
 
 def add(name, **kwargs):
@@ -218,7 +218,7 @@ def getent(refresh=False):
 
         refresh (bool):
             Refresh the info for all groups in ``__context__``. If False only
-            the groups in ``__context__`` wil be returned. If True the
+            the groups in ``__context__`` will be returned. If True the
             ``__context__`` will be refreshed with current data and returned.
             Default is False
 
@@ -469,7 +469,7 @@ def list_groups(refresh=False):
 
         refresh (bool):
             Refresh the info for all groups in ``__context__``. If False only
-            the groups in ``__context__`` wil be returned. If True, the
+            the groups in ``__context__`` will be returned. If True, the
             ``__context__`` will be refreshed with current data and returned.
             Default is False
 
