@@ -50,19 +50,18 @@ Group Policy using the ``lgpo`` module.
 :depends:
         - salt.utils.win_update
 '''
-
 # Import Python libs
-from __future__ import absolute_import
-from __future__ import unicode_literals
+from __future__ import absolute_import, unicode_literals, print_function
 import logging
 
 # Import Salt libs
-from salt.ext import six
-import salt.utils
+import salt.utils.platform
+import salt.utils.versions
 import salt.utils.win_update
 from salt.exceptions import CommandExecutionError
 
 # Import 3rd-party libs
+from salt.ext import six
 try:
     import pythoncom
     import win32com.client
@@ -77,7 +76,7 @@ def __virtual__():
     '''
     Only works on Windows systems with PyWin32
     '''
-    if not salt.utils.is_windows():
+    if not salt.utils.platform.is_windows():
         return False, 'WUA: Only available on Window systems'
 
     if not HAS_PYWIN32:
@@ -110,7 +109,7 @@ def available(software=True,
             Include software updates in the results (default is True)
 
         drivers (bool):
-            Include driver updates in the results (default is False)
+            Include driver updates in the results (default is True)
 
         summary (bool):
             - True: Return a summary of updates available for each category.
@@ -303,9 +302,9 @@ def list_update(name, download=False, install=False):
         # Not all updates have an associated KB
         salt '*' win_wua.list_update 'Microsoft Camera Codec Pack'
     '''
-    salt.utils.warn_until(
+    salt.utils.versions.warn_until(
         'Fluorine',
-        'This function is replaced by \'get\' as of Salt 2017.7.0. This'
+        'This function is replaced by \'get\' as of Salt 2017.7.0. This '
         'warning will be removed in Salt Fluorine.')
     return get(name, download, install)
 
@@ -530,9 +529,9 @@ def list_updates(software=True,
         # A summary of all Feature Packs and Windows 8.1 Updates
         salt '*' win_wua.list_updates categories=['Feature Packs','Windows 8.1'] summary=True
     '''
-    salt.utils.warn_until(
+    salt.utils.versions.warn_until(
         'Fluorine',
-        'This function is replaced by \'list\' as of Salt 2017.7.0. This'
+        'This function is replaced by \'list\' as of Salt 2017.7.0. This '
         'warning will be removed in Salt Fluorine.')
     return list(software, drivers, summary, skip_installed, categories,
                 severities, download, install)
@@ -718,9 +717,9 @@ def download_update(name):
 
         salt '*' win_wua.download_update KB12312321
     '''
-    salt.utils.warn_until(
+    salt.utils.versions.warn_until(
         'Fluorine',
-        'This function is replaced by \'download\' as of Salt 2017.7.0. This'
+        'This function is replaced by \'download\' as of Salt 2017.7.0. This '
         'warning will be removed in Salt Fluorine.')
     return download(name)
 
@@ -750,9 +749,9 @@ def download_updates(names):
         # Normal Usage
         salt '*' win_wua.download_updates guid=['12345678-abcd-1234-abcd-1234567890ab', 'KB2131233']
     '''
-    salt.utils.warn_until(
+    salt.utils.versions.warn_until(
         'Fluorine',
-        'This function is replaced by \'download\' as of Salt 2017.7.0. This'
+        'This function is replaced by \'download\' as of Salt 2017.7.0. This '
         'warning will be removed in Salt Fluorine.')
     return download(names)
 
@@ -800,7 +799,7 @@ def download(names):
         names = [names]
 
     if isinstance(names, six.integer_types):
-        names = [str(names)]
+        names = [six.text_type(names)]
 
     if updates.count() > len(names):
         raise CommandExecutionError('Multiple updates found, names need to be '
@@ -838,9 +837,9 @@ def install_update(name):
 
         salt '*' win_wua.install_update KB12312231
     '''
-    salt.utils.warn_until(
+    salt.utils.versions.warn_until(
         'Fluorine',
-        'This function is replaced by \'install\' as of Salt 2017.7.0. This'
+        'This function is replaced by \'install\' as of Salt 2017.7.0. This '
         'warning will be removed in Salt Fluorine.')
     return install(name)
 
@@ -869,9 +868,9 @@ def install_updates(names):
         # Normal Usage
         salt '*' win_wua.install_updates guid=['12345678-abcd-1234-abcd-1234567890ab', 'KB12323211']
     '''
-    salt.utils.warn_until(
+    salt.utils.versions.warn_until(
         'Fluorine',
-        'This function is replaced by \'install\' as of Salt 2017.7.0. This'
+        'This function is replaced by \'install\' as of Salt 2017.7.0. This '
         'warning will be removed in Salt Fluorine.')
     return install(names)
 
@@ -919,7 +918,7 @@ def install(names):
         names = [names]
 
     if isinstance(names, six.integer_types):
-        names = [str(names)]
+        names = [six.text_type(names)]
 
     if updates.count() > len(names):
         raise CommandExecutionError('Multiple updates found, names need to be '
