@@ -29,8 +29,8 @@ overriding the one in ``roster_defaults``:
 
 .. code-block:: yaml
 
-roster_order:
-    host: id          # use the minion id as hostname
+    roster_order:
+        host: id          # use the minion id as hostname
 
 
 You can define lists of parameters as well, the first result from the list will become the value.
@@ -41,14 +41,14 @@ Selecting a host
 
 .. code-block:: yaml
 
-# default
-roster_order:
-    host:
-      - ipv6-private  # IPv6 addresses in private ranges
-      - ipv6-global   # IPv6 addresses in global ranges
-      - ipv4-private  # IPv4 addresses in private ranges
-      - ipv4-public   # IPv4 addresses in public ranges
-      - ipv4-local    # loopback addresses
+    # default
+    roster_order:
+        host:
+          - ipv6-private  # IPv6 addresses in private ranges
+          - ipv6-global   # IPv6 addresses in global ranges
+          - ipv4-private  # IPv4 addresses in private ranges
+          - ipv4-public   # IPv4 addresses in public ranges
+          - ipv4-local    # loopback addresses
 
 
 This is the default ``roster_order``.
@@ -59,10 +59,10 @@ Other address selection parameters are also possible:
 
 .. code-block:: yaml
 
-roster_order:
-  host:
-    - global|public|private|local    # Both IPv6 and IPv4 addresses in that range
-    - 2000::/3                       # CIDR networks, both IPv4 and IPv6 are supported
+    roster_order:
+      host:
+        - global|public|private|local    # Both IPv6 and IPv4 addresses in that range
+        - 2000::/3                       # CIDR networks, both IPv4 and IPv6 are supported
 
 
 Using cached data
@@ -76,34 +76,37 @@ This should be especially useful for the other roster keys:
 
 .. code-block:: yaml
 
-roster_order:
-  host:
-    - grain: fqdn_ip4                # Lookup this grain
-    - mine: network.ip_addrs         # Mine data lookup works the same
+    roster_order:
+      host:
+        - grain: fqdn_ip4                # Lookup this grain
+        - mine: network.ip_addrs         # Mine data lookup works the same
 
-  password: sdb://vault/ssh_pass     # Salt SDB URLs are also supported
+      password: sdb://vault/ssh_pass     # Salt SDB URLs are also supported
 
-  user:
-    - pillar: ssh:auth:user          # Lookup this pillar key
-    - sdb://osenv/USER               # Lookup this env var through sdb
+      user:
+        - pillar: ssh:auth:user          # Lookup this pillar key
+        - sdb://osenv/USER               # Lookup this env var through sdb
 
-  priv:
-    - pillar:                        # Lists are also supported
-        - salt:ssh:private_key
-        - ssh:auth:private_key
+      priv:
+        - pillar:                        # Lists are also supported
+            - salt:ssh:private_key
+            - ssh:auth:private_key
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Python
 import logging
 import re
+import copy
 
 # Salt libs
+import salt.utils.data
 import salt.utils.minions
+import salt.utils.versions
 import salt.cache
 from salt._compat import ipaddress
-import salt.ext.six as six
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -116,7 +119,8 @@ def targets(tgt, tgt_type='glob', **kwargs):  # pylint: disable=W0613
     The resulting roster can be configured using ``roster_order`` and ``roster_default``.
     '''
     minions = salt.utils.minions.CkMinions(__opts__)
-    minions = minions.check_minions(tgt, tgt_type)
+    _res = minions.check_minions(tgt, tgt_type)
+    minions = _res['minions']
 
     ret = {}
     if not minions:
@@ -129,17 +133,21 @@ def targets(tgt, tgt_type='glob', **kwargs):  # pylint: disable=W0613
         'host': ('ipv6-private', 'ipv6-global', 'ipv4-private', 'ipv4-public')
     })
     if isinstance(roster_order, (tuple, list)):
-        salt.utils.warn_until('Fluorine',
-                              'Using legacy syntax for roster_order')
+        salt.utils.versions.warn_until(
+            'Fluorine',
+            'Using legacy syntax for roster_order'
+        )
         roster_order = {
             'host': roster_order
         }
     for config_key, order in roster_order.items():
         for idx, key in enumerate(order):
             if key in ('public', 'private', 'local'):
-                salt.utils.warn_until('Fluorine',
-                                      'roster_order {0} will include IPv6 soon. '
-                                      'Set order to ipv4-{0} if needed.'.format(key))
+                salt.utils.versions.warn_until(
+                    'Fluorine',
+                    'roster_order {0} will include IPv6 soon. '
+                    'Set order to ipv4-{0} if needed.'.format(key)
+                )
                 order[idx] = 'ipv4-' + key
 
     # log.debug(roster_order)
@@ -151,7 +159,7 @@ def targets(tgt, tgt_type='glob', **kwargs):  # pylint: disable=W0613
         except LookupError:
             continue
 
-        minion_res = __opts__.get('roster_defaults', {}).copy()
+        minion_res = copy.deepcopy(__opts__.get('roster_defaults', {}))
         for param, order in roster_order.items():
             if not isinstance(order, (list, tuple)):
                 order = [order]
@@ -164,9 +172,9 @@ def targets(tgt, tgt_type='glob', **kwargs):  # pylint: disable=W0613
         if 'host' in minion_res:
             ret[minion_id] = minion_res
         else:
-            log.warning('Could not determine host information for minion {0}'.format(minion_id))
+            log.warning('Could not determine host information for minion %s', minion_id)
 
-    log.debug('Roster lookup result: {0}'.format(ret))
+    log.debug('Roster lookup result: %s', ret)
 
     return ret
 
@@ -175,15 +183,15 @@ def _load_minion(minion_id, cache):
     data_minion, grains, pillar = salt.utils.minions.get_minion_data(minion_id, __opts__)
 
     if minion_id != data_minion:
-        log.error('Asked for minion {0}, got {1}'.format(minion_id, data_minion))
+        log.error('Asked for minion %s, got %s', minion_id, data_minion)
         raise LookupError
 
     if not grains:
-        log.warning('No grain data for minion id {0}'.format(minion_id))
+        log.warning('No grain data for minion id %s', minion_id)
         grains = {}
 
     if not pillar:
-        log.warning('No pillar data for minion id {0}'.format(minion_id))
+        log.warning('No pillar data for minion id %s', minion_id)
         pillar = {}
 
     addrs = {
@@ -202,8 +210,8 @@ def _data_lookup(ref, lookup):
 
     res = []
     for data_key in lookup:
-        data = salt.utils.traverse_dict_and_list(ref, data_key, None)
-        # log.debug('Fetched {0} in {1}: {2}'.format(data_key, ref, data))
+        data = salt.utils.data.traverse_dict_and_list(ref, data_key, None)
+        # log.debug('Fetched %s in %s: %s', data_key, ref, data)
         if data:
             res.append(data)
 
@@ -238,12 +246,12 @@ def _minion_lookup(minion_id, key, minion):
         try:
             net = ipaddress.ip_network(key, strict=True)
         except ValueError:
-            log.error('{0} is an invalid CIDR network'.format(net))
+            log.error('%s is an invalid CIDR network', net)
             return None
 
         for addr in addrs[net.version]:
             if addr in net:
-                return str(addr)
+                return six.text_type(addr)
     else:
         # Take the addresses from the grains and filter them
         filters = {
@@ -262,6 +270,6 @@ def _minion_lookup(minion_id, key, minion):
             try:
                 for addr in addrs[ip_ver]:
                     if filters[key](addr):
-                        return str(addr)
+                        return six.text_type(addr)
             except KeyError:
                 raise KeyError('Invalid filter {0} specified in roster_order'.format(key))
