@@ -5,13 +5,13 @@ http://stackoverflow.com/a/3233356
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import collections
 
 # Import 3rd-party libs
 import copy
 import logging
-import salt.ext.six as six
+from salt.ext import six
 from salt.serializers.yamlex import merge_recursive as _yamlex_merge_recursive
 
 log = logging.getLogger(__name__)
@@ -65,7 +65,7 @@ def update(dest, upd, recursive_update=True, merge_lists=False):
         return dest
     else:
         try:
-            for k in upd.keys():
+            for k in upd:
                 dest[k] = upd[k]
         except AttributeError:
             # this mapping is not a dict
@@ -121,8 +121,10 @@ def merge(obj_a, obj_b, strategy='smart', renderer='yaml', merge_lists=False):
         # we just do not want to log an error
         merged = merge_recurse(obj_a, obj_b)
     else:
-        log.warning('Unknown merging strategy \'{0}\', '
-                    'fallback to recurse'.format(strategy))
+        log.warning(
+            'Unknown merging strategy \'%s\', fallback to recurse',
+            strategy
+        )
         merged = merge_recurse(obj_a, obj_b)
 
     return merged
