@@ -4,16 +4,20 @@ Define some generic socket functions for network modules
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
+import itertools
 import os
 import re
+import types
 import socket
 import logging
 import platform
+import random
+import subprocess
 from string import ascii_letters, digits
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 # Attempt to import wmi
 try:
@@ -23,15 +27,30 @@ except ImportError:
     pass
 
 # Import salt libs
-import salt.utils
-from salt._compat import subprocess, ipaddress
+import salt.utils.args
+import salt.utils.files
+import salt.utils.path
+import salt.utils.platform
+import salt.utils.stringutils
+import salt.utils.zeromq
+from salt._compat import ipaddress
+from salt.exceptions import SaltClientError, SaltSystemExit
+from salt.utils.decorators.jinja import jinja_filter
 
 # inet_pton does not exist in Windows, this is a workaround
-if salt.utils.is_windows():
+if salt.utils.platform.is_windows():
     from salt.ext import win_inet_pton  # pylint: disable=unused-import
 
 log = logging.getLogger(__name__)
 
+try:
+    import ctypes
+    import ctypes.util
+    libc = ctypes.cdll.LoadLibrary(ctypes.util.find_library("c"))
+    res_init = libc.__res_init
+except (ImportError, OSError, AttributeError, TypeError):
+    pass
+
 # pylint: disable=C0103
 
 
@@ -120,26 +139,26 @@ def _generate_minion_id():
     hosts = DistinctList().append(socket.getfqdn()).append(platform.node()).append(socket.gethostname())
     if not hosts:
         try:
-            for a_nfo in socket.getaddrinfo(hosts.first(), None, socket.AF_INET,
+            for a_nfo in socket.getaddrinfo(hosts.first() or 'localhost', None, socket.AF_INET,
                                             socket.SOCK_RAW, socket.IPPROTO_IP, socket.AI_CANONNAME):
                 if len(a_nfo) > 3:
                     hosts.append(a_nfo[3])
         except socket.gaierror:
             log.warning('Cannot resolve address {addr} info via socket: {message}'.format(
-                addr=hosts.first(), message=socket.gaierror)
+                addr=hosts.first() or 'localhost (N/A)', message=socket.gaierror)
             )
     # Universal method for everywhere (Linux, Slowlaris, Windows etc)
     for f_name in ['/etc/hostname', '/etc/nodename', '/etc/hosts',
                    r'{win}\system32\drivers\etc\hosts'.format(win=os.getenv('WINDIR'))]:
         if not os.path.exists(f_name):
             continue
-        with salt.utils.fopen(f_name) as f_hdl:
+        with salt.utils.files.fopen(f_name) as f_hdl:
             for hst in (line.strip().split('#')[0].strip().split() or None for line in f_hdl.read().split(os.linesep)):
                 if hst and (hst[0][:4] in ['127.', '::1'] or len(hst) == 1):
                     hosts.extend(hst)
 
     # include public and private ipaddresses
-    return hosts.extend([addr for addr in salt.utils.network.ip_addrs()
+    return hosts.extend([addr for addr in ip_addrs()
                          if not ipaddress.ip_address(addr).is_loopback])
 
 
@@ -149,7 +168,11 @@ def generate_minion_id():
 
     :return:
     '''
-    return _generate_minion_id().first() or 'localhost'
+    try:
+        ret = salt.utils.stringutils.to_unicode(_generate_minion_id().first())
+    except TypeError:
+        ret = None
+    return ret or 'localhost'
 
 
 def get_socket(addr, type=socket.SOCK_STREAM, proto=0):
@@ -195,13 +218,29 @@ def ip_to_host(ip):
     '''
     try:
         hostname, aliaslist, ipaddrlist = socket.gethostbyaddr(ip)
-    except Exception:
+    except Exception as exc:
+        log.debug('salt.utils.network.ip_to_host(%r) failed: %s', ip, exc)
         hostname = None
     return hostname
 
 # pylint: enable=C0103
 
 
+def is_reachable_host(entity_name):
+    '''
+    Returns a bool telling if the entity name is a reachable host (IPv4/IPv6/FQDN/etc).
+    :param hostname:
+    :return:
+    '''
+    try:
+        assert type(socket.getaddrinfo(entity_name, 0, 0, 0, 0)) == list
+        ret = True
+    except socket.gaierror:
+        ret = False
+
+    return ret
+
+
 def is_ip(ip):
     '''
     Returns a bool telling if the passed IP is a valid IPv4 or IPv6 address.
@@ -229,6 +268,329 @@ def is_ipv6(ip):
         return False
 
 
+def is_subnet(cidr):
+    '''
+    Returns a bool telling if the passed string is an IPv4 or IPv6 subnet
+    '''
+    return is_ipv4_subnet(cidr) or is_ipv6_subnet(cidr)
+
+
+def is_ipv4_subnet(cidr):
+    '''
+    Returns a bool telling if the passed string is an IPv4 subnet
+    '''
+    try:
+        return '/' in cidr and bool(ipaddress.IPv4Network(cidr))
+    except Exception:
+        return False
+
+
+def is_ipv6_subnet(cidr):
+    '''
+    Returns a bool telling if the passed string is an IPv6 subnet
+    '''
+    try:
+        return '/' in cidr and bool(ipaddress.IPv6Network(cidr))
+    except Exception:
+        return False
+
+
+@jinja_filter('is_ip')
+def is_ip_filter(ip, options=None):
+    '''
+    Returns a bool telling if the passed IP is a valid IPv4 or IPv6 address.
+    '''
+    return is_ipv4_filter(ip, options=options) or is_ipv6_filter(ip, options=options)
+
+
+def _ip_options_global(ip_obj, version):
+    return not ip_obj.is_private
+
+
+def _ip_options_multicast(ip_obj, version):
+    return ip_obj.is_multicast
+
+
+def _ip_options_loopback(ip_obj, version):
+    return ip_obj.is_loopback
+
+
+def _ip_options_link_local(ip_obj, version):
+    return ip_obj.is_link_local
+
+
+def _ip_options_private(ip_obj, version):
+    return ip_obj.is_private
+
+
+def _ip_options_reserved(ip_obj, version):
+    return ip_obj.is_reserved
+
+
+def _ip_options_site_local(ip_obj, version):
+    if version == 6:
+        return ip_obj.is_site_local
+    return False
+
+
+def _ip_options_unspecified(ip_obj, version):
+    return ip_obj.is_unspecified
+
+
+def _ip_options(ip_obj, version, options=None):
+
+    # will process and IP options
+    options_fun_map = {
+        'global': _ip_options_global,
+        'link-local': _ip_options_link_local,
+        'linklocal': _ip_options_link_local,
+        'll': _ip_options_link_local,
+        'link_local': _ip_options_link_local,
+        'loopback': _ip_options_loopback,
+        'lo': _ip_options_loopback,
+        'multicast': _ip_options_multicast,
+        'private': _ip_options_private,
+        'public': _ip_options_global,
+        'reserved': _ip_options_reserved,
+        'site-local': _ip_options_site_local,
+        'sl': _ip_options_site_local,
+        'site_local': _ip_options_site_local,
+        'unspecified': _ip_options_unspecified
+    }
+
+    if not options:
+        return six.text_type(ip_obj)  # IP version already checked
+
+    options_list = [option.strip() for option in options.split(',')]
+
+    for option, fun in options_fun_map.items():
+        if option in options_list:
+            fun_res = fun(ip_obj, version)
+            if not fun_res:
+                return None
+                # stop at first failed test
+            # else continue
+    return six.text_type(ip_obj)
+
+
+def _is_ipv(ip, version, options=None):
+
+    if not version:
+        version = 4
+
+    if version not in (4, 6):
+        return None
+
+    try:
+        ip_obj = ipaddress.ip_address(ip)
+    except ValueError:
+        # maybe it is an IP network
+        try:
+            ip_obj = ipaddress.ip_interface(ip)
+        except ValueError:
+            # nope, still not :(
+            return None
+
+    if not ip_obj.version == version:
+        return None
+
+    # has the right version, let's move on
+    return _ip_options(ip_obj, version, options=options)
+
+
+@jinja_filter('is_ipv4')
+def is_ipv4_filter(ip, options=None):
+    '''
+    Returns a bool telling if the value passed to it was a valid IPv4 address.
+
+    ip
+        The IP address.
+
+    net: False
+        Consider IP addresses followed by netmask.
+
+    options
+        CSV of options regarding the nature of the IP address. E.g.: loopback, multicast, private etc.
+    '''
+    _is_ipv4 = _is_ipv(ip, 4, options=options)
+    return isinstance(_is_ipv4, six.string_types)
+
+
+@jinja_filter('is_ipv6')
+def is_ipv6_filter(ip, options=None):
+    '''
+    Returns a bool telling if the value passed to it was a valid IPv6 address.
+
+    ip
+        The IP address.
+
+    net: False
+        Consider IP addresses followed by netmask.
+
+    options
+        CSV of options regarding the nature of the IP address. E.g.: loopback, multicast, private etc.
+    '''
+    _is_ipv6 = _is_ipv(ip, 6, options=options)
+    return isinstance(_is_ipv6, six.string_types)
+
+
+def _ipv_filter(value, version, options=None):
+
+    if version not in (4, 6):
+        return
+
+    if isinstance(value, (six.string_types, six.text_type, six.binary_type)):
+        return _is_ipv(value, version, options=options)  # calls is_ipv4 or is_ipv6 for `value`
+    elif isinstance(value, (list, tuple, types.GeneratorType)):
+        # calls is_ipv4 or is_ipv6 for each element in the list
+        # os it filters and returns only those elements having the desired IP version
+        return [
+            _is_ipv(addr, version, options=options)
+            for addr in value
+            if _is_ipv(addr, version, options=options) is not None
+        ]
+    return None
+
+
+@jinja_filter('ipv4')
+def ipv4(value, options=None):
+    '''
+    Filters a list and returns IPv4 values only.
+    '''
+    return _ipv_filter(value, 4, options=options)
+
+
+@jinja_filter('ipv6')
+def ipv6(value, options=None):
+    '''
+    Filters a list and returns IPv6 values only.
+    '''
+    return _ipv_filter(value, 6, options=options)
+
+
+@jinja_filter('ipaddr')
+def ipaddr(value, options=None):
+    '''
+    Filters and returns only valid IP objects.
+    '''
+    ipv4_obj = ipv4(value, options=options)
+    ipv6_obj = ipv6(value, options=options)
+    if ipv4_obj is None or ipv6_obj is None:
+        # an IP address can be either IPv4 either IPv6
+        # therefofe if the value passed as arg is not a list, at least one of the calls above will return None
+        # if one of them is none, means that we should return only one of them
+        return ipv4_obj or ipv6_obj  # one of them
+    else:
+        return ipv4_obj + ipv6_obj  # extend lists
+
+
+def _filter_ipaddr(value, options, version=None):
+    ipaddr_filter_out = None
+    if version:
+        if version == 4:
+            ipaddr_filter_out = ipv4(value, options)
+        elif version == 6:
+            ipaddr_filter_out = ipv6(value, options)
+    else:
+        ipaddr_filter_out = ipaddr(value, options)
+    if not ipaddr_filter_out:
+        return
+    if not isinstance(ipaddr_filter_out, (list, tuple, types.GeneratorType)):
+        ipaddr_filter_out = [ipaddr_filter_out]
+    return ipaddr_filter_out
+
+
+@jinja_filter('ip_host')
+def ip_host(value, options=None, version=None):
+    '''
+    Returns the interfaces IP address, e.g.: 192.168.0.1/28.
+    '''
+    ipaddr_filter_out = _filter_ipaddr(value, options=options, version=version)
+    if not ipaddr_filter_out:
+        return
+    if not isinstance(value, (list, tuple, types.GeneratorType)):
+        return six.text_type(ipaddress.ip_interface(ipaddr_filter_out[0]))
+    return [six.text_type(ipaddress.ip_interface(ip_a)) for ip_a in ipaddr_filter_out]
+
+
+def _network_hosts(ip_addr_entry):
+    return [
+        six.text_type(host)
+        for host in ipaddress.ip_network(ip_addr_entry, strict=False).hosts()
+    ]
+
+
+@jinja_filter('network_hosts')
+def network_hosts(value, options=None, version=None):
+    '''
+    Return the list of hosts within a network.
+
+    .. note::
+
+        When running this command with a large IPv6 network, the command will
+        take a long time to gather all of the hosts.
+    '''
+    ipaddr_filter_out = _filter_ipaddr(value, options=options, version=version)
+    if not ipaddr_filter_out:
+        return
+    if not isinstance(value, (list, tuple, types.GeneratorType)):
+        return _network_hosts(ipaddr_filter_out[0])
+    return [
+        _network_hosts(ip_a)
+        for ip_a in ipaddr_filter_out
+    ]
+
+
+def _network_size(ip_addr_entry):
+    return ipaddress.ip_network(ip_addr_entry, strict=False).num_addresses
+
+
+@jinja_filter('network_size')
+def network_size(value, options=None, version=None):
+    '''
+    Get the size of a network.
+    '''
+    ipaddr_filter_out = _filter_ipaddr(value, options=options, version=version)
+    if not ipaddr_filter_out:
+        return
+    if not isinstance(value, (list, tuple, types.GeneratorType)):
+        return _network_size(ipaddr_filter_out[0])
+    return [
+        _network_size(ip_a)
+        for ip_a in ipaddr_filter_out
+    ]
+
+
+def natural_ipv4_netmask(ip, fmt='prefixlen'):
+    '''
+    Returns the "natural" mask of an IPv4 address
+    '''
+    bits = _ipv4_to_bits(ip)
+
+    if bits.startswith('11'):
+        mask = '24'
+    elif bits.startswith('1'):
+        mask = '16'
+    else:
+        mask = '8'
+
+    if fmt == 'netmask':
+        return cidr_to_ipv4_netmask(mask)
+    else:
+        return '/' + mask
+
+
+def rpad_ipv4_network(ip):
+    '''
+    Returns an IP network address padded with zeros.
+
+    Ex: '192.168.3' -> '192.168.3.0'
+        '10.209' -> '10.209.0.0'
+    '''
+    return '.'.join(itertools.islice(itertools.chain(ip.split('.'), '0000'), 0,
+                                     4))
+
+
 def cidr_to_ipv4_netmask(cidr_bits):
     '''
     Returns an IPv4 netmask
@@ -380,7 +742,7 @@ def _interfaces_ifconfig(out):
 
     piface = re.compile(r'^([^\s:]+)')
     pmac = re.compile('.*?(?:HWaddr|ether|address:|lladdr) ([0-9a-fA-F:]+)')
-    if salt.utils.is_sunos():
+    if salt.utils.platform.is_sunos():
         pip = re.compile(r'.*?(?:inet\s+)([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)(.*)')
         pip6 = re.compile('.*?(?:inet6 )([0-9a-fA-F:]+)')
         pmask6 = re.compile(r'.*?(?:inet6 [0-9a-fA-F:]+/(\d+)).*')
@@ -407,7 +769,7 @@ def _interfaces_ifconfig(out):
                 iface = miface.group(1)
             if mmac:
                 data['hwaddr'] = mmac.group(1)
-                if salt.utils.is_sunos():
+                if salt.utils.platform.is_sunos():
                     expand_mac = []
                     for chunk in data['hwaddr'].split(':'):
                         expand_mac.append('0{0}'.format(chunk) if len(chunk) < 2 else '{0}'.format(chunk))
@@ -439,11 +801,11 @@ def _interfaces_ifconfig(out):
                 mmask6 = pmask6.match(line)
                 if mmask6:
                     addr_obj['prefixlen'] = mmask6.group(1) or mmask6.group(2)
-                    if not salt.utils.is_sunos():
+                    if not salt.utils.platform.is_sunos():
                         ipv6scope = mmask6.group(3) or mmask6.group(4)
                         addr_obj['scope'] = ipv6scope.lower() if ipv6scope is not None else ipv6scope
                 # SunOS sometimes has ::/0 as inet6 addr when using addrconf
-                if not salt.utils.is_sunos() \
+                if not salt.utils.platform.is_sunos() \
                         or addr_obj['address'] != '::' \
                         and addr_obj['prefixlen'] != 0:
                     data['inet6'].append(addr_obj)
@@ -472,8 +834,8 @@ def linux_interfaces():
     Obtain interface information for *NIX/BSD variants
     '''
     ifaces = dict()
-    ip_path = salt.utils.which('ip')
-    ifconfig_path = None if ip_path else salt.utils.which('ifconfig')
+    ip_path = salt.utils.path.which('ip')
+    ifconfig_path = None if ip_path else salt.utils.path.which('ifconfig')
     if ip_path:
         cmd1 = subprocess.Popen(
             '{0} link show'.format(ip_path),
@@ -488,15 +850,15 @@ def linux_interfaces():
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT).communicate()[0]
         ifaces = _interfaces_ip("{0}\n{1}".format(
-            salt.utils.to_str(cmd1),
-            salt.utils.to_str(cmd2)))
+            salt.utils.stringutils.to_str(cmd1),
+            salt.utils.stringutils.to_str(cmd2)))
     elif ifconfig_path:
         cmd = subprocess.Popen(
             '{0} -a'.format(ifconfig_path),
             shell=True,
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT).communicate()[0]
-        ifaces = _interfaces_ifconfig(salt.utils.to_str(cmd))
+        ifaces = _interfaces_ifconfig(salt.utils.stringutils.to_str(cmd))
     return ifaces
 
 
@@ -599,7 +961,7 @@ def interfaces():
     '''
     Return a dictionary of information about all the interfaces on the minion
     '''
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return win_interfaces()
     else:
         return linux_interfaces()
@@ -610,7 +972,7 @@ def get_net_start(ipaddr, netmask):
     Return the address of the network
     '''
     net = ipaddress.ip_network('{0}/{1}'.format(ipaddr, netmask), strict=False)
-    return str(net.network_address)
+    return six.text_type(net.network_address)
 
 
 def get_net_size(mask):
@@ -633,7 +995,7 @@ def calc_net(ipaddr, netmask=None):
     if netmask is not None:
         ipaddr = '{0}/{1}'.format(ipaddr, netmask)
 
-    return str(ipaddress.ip_network(ipaddr, strict=False))
+    return six.text_type(ipaddress.ip_network(ipaddr, strict=False))
 
 
 def _ipv4_to_bits(ipaddr):
@@ -690,7 +1052,7 @@ def hw_addr(iface):
         Added support for AIX
 
     '''
-    if salt.utils.is_aix():
+    if salt.utils.platform.is_aix():
         return _hw_addr_aix
 
     iface_info, error = _get_iface_info(iface)
@@ -720,7 +1082,8 @@ def interface_ip(iface):
     iface_info, error = _get_iface_info(iface)
 
     if error is False:
-        return iface_info.get(iface, {}).get('inet', {})[0].get('address', '')
+        inet = iface_info.get(iface, {}).get('inet', None)
+        return inet[0].get('address', '') if inet else ''
     else:
         return error
 
@@ -762,7 +1125,7 @@ def _subnets(proto='inet', interfaces_=None):
                 intf = ipaddress.ip_interface('{0}/{1}'.format(intf['address'], dflt_cidr))
             if not intf.is_loopback:
                 ret.add(intf.network)
-    return [str(net) for net in sorted(ret)]
+    return [six.text_type(net) for net in sorted(ret)]
 
 
 def subnets(interfaces=None):
@@ -827,7 +1190,7 @@ def _ip_addrs(interface=None, include_loopback=False, interface_data=None, proto
             addr = ipaddress.ip_address(addr.get('address'))
             if not addr.is_loopback or include_loopback:
                 ret.add(addr)
-    return [str(addr) for addr in sorted(ret)]
+    return [six.text_type(addr) for addr in sorted(ret)]
 
 
 def ip_addrs(interface=None, include_loopback=False, interface_data=None):
@@ -911,8 +1274,9 @@ def active_tcp():
     ret = {}
     for statf in ['/proc/net/tcp', '/proc/net/tcp6']:
         if os.path.isfile(statf):
-            with salt.utils.fopen(statf, 'rb') as fp_:
+            with salt.utils.files.fopen(statf, 'rb') as fp_:
                 for line in fp_:
+                    line = salt.utils.stringutils.to_unicode(line)
                     if line.strip().startswith('sl'):
                         continue
                     ret.update(_parse_tcp_line(line))
@@ -946,8 +1310,9 @@ def _remotes_on(port, which_end):
     for statf in ['/proc/net/tcp', '/proc/net/tcp6']:
         if os.path.isfile(statf):
             proc_available = True
-            with salt.utils.fopen(statf, 'r') as fp_:
+            with salt.utils.files.fopen(statf, 'r') as fp_:
                 for line in fp_:
+                    line = salt.utils.stringutils.to_unicode(line)
                     if line.strip().startswith('sl'):
                         continue
                     iret = _parse_tcp_line(line)
@@ -956,17 +1321,17 @@ def _remotes_on(port, which_end):
                         ret.add(iret[sl]['remote_addr'])
 
     if not proc_available:  # Fallback to use OS specific tools
-        if salt.utils.is_sunos():
+        if salt.utils.platform.is_sunos():
             return _sunos_remotes_on(port, which_end)
-        if salt.utils.is_freebsd():
+        if salt.utils.platform.is_freebsd():
             return _freebsd_remotes_on(port, which_end)
-        if salt.utils.is_netbsd():
+        if salt.utils.platform.is_netbsd():
             return _netbsd_remotes_on(port, which_end)
-        if salt.utils.is_openbsd():
+        if salt.utils.platform.is_openbsd():
             return _openbsd_remotes_on(port, which_end)
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             return _windows_remotes_on(port, which_end)
-        if salt.utils.is_aix():
+        if salt.utils.platform.is_aix():
             return _aix_remotes_on(port, which_end)
 
         return _linux_remotes_on(port, which_end)
@@ -1013,7 +1378,7 @@ def _sunos_remotes_on(port, which_end):
         log.error('Failed netstat')
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
     for line in lines:
         if 'ESTABLISHED' not in line:
             continue
@@ -1053,13 +1418,13 @@ def _freebsd_remotes_on(port, which_end):
     remotes = set()
 
     try:
-        cmd = salt.utils.shlex_split('sockstat -4 -c -p {0}'.format(port))
+        cmd = salt.utils.args.shlex_split('sockstat -4 -c -p {0}'.format(port))
         data = subprocess.check_output(cmd)  # pylint: disable=minimum-python-version
     except subprocess.CalledProcessError as ex:
         log.error('Failed "sockstat" with returncode = {0}'.format(ex.returncode))
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
 
     for line in lines:
         chunks = line.split()
@@ -1113,13 +1478,13 @@ def _netbsd_remotes_on(port, which_end):
     remotes = set()
 
     try:
-        cmd = salt.utils.shlex_split('sockstat -4 -c -n -p {0}'.format(port))
+        cmd = salt.utils.args.shlex_split('sockstat -4 -c -n -p {0}'.format(port))
         data = subprocess.check_output(cmd)  # pylint: disable=minimum-python-version
     except subprocess.CalledProcessError as ex:
         log.error('Failed "sockstat" with returncode = {0}'.format(ex.returncode))
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
 
     for line in lines:
         chunks = line.split()
@@ -1208,7 +1573,7 @@ def _windows_remotes_on(port, which_end):
         log.error('Failed netstat')
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
     for line in lines:
         if 'ESTABLISHED' not in line:
             continue
@@ -1255,7 +1620,7 @@ def _linux_remotes_on(port, which_end):
         log.error('Failed "lsof" with returncode = {0}'.format(ex.returncode))
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
     for line in lines:
         chunks = line.split()
         if not chunks:
@@ -1314,7 +1679,7 @@ def _aix_remotes_on(port, which_end):
         log.error('Failed netstat')
         raise
 
-    lines = salt.utils.to_str(data).split('\n')
+    lines = salt.utils.stringutils.to_str(data).split('\n')
     for line in lines:
         if 'ESTABLISHED' not in line:
             continue
@@ -1328,3 +1693,130 @@ def _aix_remotes_on(port, which_end):
             continue
         remotes.add(remote_host)
     return remotes
+
+
+@jinja_filter('gen_mac')
+def gen_mac(prefix='AC:DE:48'):
+    '''
+    Generates a MAC address with the defined OUI prefix.
+
+    Common prefixes:
+
+     - ``00:16:3E`` -- Xen
+     - ``00:18:51`` -- OpenVZ
+     - ``00:50:56`` -- VMware (manually generated)
+     - ``52:54:00`` -- QEMU/KVM
+     - ``AC:DE:48`` -- PRIVATE
+
+    References:
+
+     - http://standards.ieee.org/develop/regauth/oui/oui.txt
+     - https://www.wireshark.org/tools/oui-lookup.html
+     - https://en.wikipedia.org/wiki/MAC_address
+    '''
+    return '{0}:{1:02X}:{2:02X}:{3:02X}'.format(prefix,
+                                                random.randint(0, 0xff),
+                                                random.randint(0, 0xff),
+                                                random.randint(0, 0xff))
+
+
+@jinja_filter('mac_str_to_bytes')
+def mac_str_to_bytes(mac_str):
+    '''
+    Convert a MAC address string into bytes. Works with or without separators:
+
+    b1 = mac_str_to_bytes('08:00:27:13:69:77')
+    b2 = mac_str_to_bytes('080027136977')
+    assert b1 == b2
+    assert isinstance(b1, bytes)
+    '''
+    if len(mac_str) == 12:
+        pass
+    elif len(mac_str) == 17:
+        sep = mac_str[2]
+        mac_str = mac_str.replace(sep, '')
+    else:
+        raise ValueError('Invalid MAC address')
+    chars = (int(mac_str[s:s+2], 16) for s in range(0, 12, 2))
+    return bytes(chars) if six.PY3 else b''.join(chr(x) for x in chars)
+
+
+def refresh_dns():
+    '''
+    issue #21397: force glibc to re-read resolv.conf
+    '''
+    try:
+        res_init()
+    except NameError:
+        # Exception raised loading the library, thus res_init is not defined
+        pass
+
+
+@jinja_filter('dns_check')
+def dns_check(addr, port, safe=False, ipv6=None):
+    '''
+    Return the ip resolved by dns, but do not exit on failure, only raise an
+    exception. Obeys system preference for IPv4/6 address resolution.
+    Tries to connect to the address before considering it useful. If no address
+    can be reached, the first one resolved is used as a fallback.
+    '''
+    error = False
+    lookup = addr
+    seen_ipv6 = False
+    try:
+        refresh_dns()
+        hostnames = socket.getaddrinfo(
+            addr, None, socket.AF_UNSPEC, socket.SOCK_STREAM
+        )
+        if not hostnames:
+            error = True
+        else:
+            resolved = False
+            candidates = []
+            for h in hostnames:
+                # It's an IP address, just return it
+                if h[4][0] == addr:
+                    resolved = addr
+                    break
+
+                if h[0] == socket.AF_INET and ipv6 is True:
+                    continue
+                if h[0] == socket.AF_INET6 and ipv6 is False:
+                    continue
+
+                candidate_addr = salt.utils.zeromq.ip_bracket(h[4][0])
+
+                if h[0] != socket.AF_INET6 or ipv6 is not None:
+                    candidates.append(candidate_addr)
+
+                try:
+                    s = socket.socket(h[0], socket.SOCK_STREAM)
+                    s.connect((candidate_addr.strip('[]'), port))
+                    s.close()
+
+                    resolved = candidate_addr
+                    break
+                except socket.error:
+                    pass
+            if not resolved:
+                if len(candidates) > 0:
+                    resolved = candidates[0]
+                else:
+                    error = True
+    except TypeError:
+        err = ('Attempt to resolve address \'{0}\' failed. Invalid or unresolveable address').format(lookup)
+        raise SaltSystemExit(code=42, msg=err)
+    except socket.error:
+        error = True
+
+    if error:
+        err = ('DNS lookup or connection check of \'{0}\' failed.').format(addr)
+        if safe:
+            if salt.log.is_console_configured():
+                # If logging is not configured it also means that either
+                # the master or minion instance calling this hasn't even
+                # started running
+                log.error(err)
+            raise SaltClientError()
+        raise SaltSystemExit(code=42, msg=err)
+    return resolved
